/*
 * This file stands for Graphics processor unit. 
 * It draws only the text-mode # 0 now.
 */
import java.awt.*;
public class Antic implements Colors, AtariConstants {
  AtariMem memory;
  Color atari_colortable[] = new Color[colortable.length];
  boolean NMI_MASK = false;
  int dlicount;
  int screenaddr;
  int ypos;
  int xmin, xmax, scroll_offset;
  int[] pf_scanline = new int[ATARI_WIDTH];
  int[] lookup1 = new int[256];
  int[] lookup2 = new int[256];

  public Antic (AtariMem mem) {
    int rgb;
    int red, green, blue;
      installMem (mem);

    /*
       for (int i=0; i<colortable.length; i++) {
       rgb = colortable[i];

       red = (rgb & 0xff0000) >> 16;
       green = (rgb & 0x00ff00) >> 8;
       blue = (rgb & 0x0000ff);

       atari_colortable[i] = new Color((float)red/256, (float)green/256, (float)blue/256 );
       }
     */
  } public void installMem (AtariMem mem) {
    memory = mem;
  }

/**
 * Antic has its own program which tells him what he could do.
 * This programm we call Display List. It's vector is at address
 * 560,561.
 *
 * @return     Returns the display list pointer. 
 */
  public int getDLAddress () {
    return (memory.getWord (560));
  }

  public void printDL () {
    int i;
    int DLBase = getDLAddress ();
    System.out.println ("Display List program @ " + DLBase);
    for (i = 0; i < 120; i++) {
      System.out.print (memory.mem[DLBase + i] + " ");
    }
    System.out.println ("");
  }

  public void UpdateScreen (Graphics g) {
    int dlist;
    boolean JVB;
    int IR;
    int nlines;

    //scrn_ptr = screen;
    //PM_InitFrame ();
    ypos = 0;
    if ((memory.hwreg.DMACTL & 0x20) != 0) {
      dlist = getDLAddress ();
      JVB = false;

      do {
	IR = memory.getByte (dlist++);
	if ((IR & memory.hwreg.NMIEN & 0x80) != 0)
	  dlicount = 1;

	else
	  dlicount = 0;
	switch (IR & 0x0f) {
	case 0x00:
	  nlines = ((IR >> 4) & 0x07) + 1;
	  antic_blank (g, nlines);
	  break;
	case 0x01:
	  if ((IR & 0x40) != 0) {
	    JVB = true;
	  }
	  else {
	    dlist = memory.getWord (dlist);
	    antic_blank (g, 1);	/* Jump aparently uses 1 scan line */
	  }
	  break;
	default:
	  if ((IR & 0x40) != 0) {
	    screenaddr = memory.getWord (dlist);
	    dlist += 2;
	  }
	  if ((IR & 0x20) != 0)
	    System.out.println ("DLIST: vertical scroll unsupported");
	  if ((IR & 0x10) != 0) {
	    xmin = memory.dmactl_xmin_scroll;
	    xmax = memory.dmactl_xmax_scroll;
	    scroll_offset = memory.hwreg.HSCROL << 1;
	  }
	  else {
	    xmin = memory.dmactl_xmin_noscroll;
	    xmax = memory.dmactl_xmax_noscroll;
	    scroll_offset = 0;
	  }
	  switch (IR & 0x0f) {
	  case 0x02:
	    antic_2text (g);
	    break;
	  case 0x03:
	    antic_3 ();
	    break;
	  case 0x04:
	    antic_4 ();
	    break;
	  case 0x05:
	    antic_5 ();
	    break;
	  case 0x06:
	    antic_6 ();
	    break;
	  case 0x07:
	    antic_7 ();
	    break;
	  case 0x08:
	    antic_8 ();
	    break;
	  case 0x09:
	    antic_9 ();
	    break;
	  case 0x0a:
	    antic_a ();
	    break;
	  case 0x0b:
	    antic_b ();
	    break;
	  case 0x0c:
	    antic_c ();
	    break;
	  case 0x0d:
	    antic_d ();
	    break;
	  case 0x0e:
	    antic_e ();
	    break;
	  case 0x0f:
	    antic_f ();
	    break;
	  default:
	    JVB = true;
	    break;
	  }
	  break;
	}
      }
      while (!JVB && (ypos < ATARI_HEIGHT));
    }
    dlicount = 0;
    antic_blank (g, ATARI_HEIGHT - ypos);

    //Atari_DisplayScreen (screen);
  }

  public void antic_blank (Graphics g, int nlines) {
    g.setColor (Color.black);
    for (int i = 0; i < nlines; i++) {
      g.drawLine (0, ypos + 24, ATARI_WIDTH, ypos + 24);
      ypos++;
    }
  }

  public void antic_2text (Graphics g) {
    int nchars;
    int row;
    int[] scr_char = new int[40];
    int[] char_addr = new int[40];
    int font_data;
    int xpos;
    nchars = (xmax - xmin) >> 3;	// 40 characters per line
    lookup1[0x80] = lookup1[0x40] = lookup1[0x20] = lookup1[0x10] =
      lookup1[0x08] = lookup1[0x04] = lookup1[0x02] = lookup1[0x01] = 0x0002;
    lookup1[0x00] = 0x0004;
    for (row = 0; row < nchars; row++) {
      scr_char[row] = memory.getByte (screenaddr++);
      char_addr[row] = memory.chbase_40 + ((scr_char[row] & 0x7f) << 3);
    }
    for (int i = 0; i < 8; i++) {
      xpos = xmin;
      for (row = 0; row < nchars; row++) {
	if (scr_char[row] != 0) {
	  font_data = memory.getByte (char_addr[row] + i);
	  if ((scr_char[row] & 0x80) != 0)
	    font_data ^= 0xff;
	  pf_scanline[xpos++] = lookup1[font_data & 0x80];
	  pf_scanline[xpos++] = lookup1[font_data & 0x40];
	  pf_scanline[xpos++] = lookup1[font_data & 0x20];
	  pf_scanline[xpos++] = lookup1[font_data & 0x10];
	  pf_scanline[xpos++] = lookup1[font_data & 0x08];
	  pf_scanline[xpos++] = lookup1[font_data & 0x04];
	  pf_scanline[xpos++] = lookup1[font_data & 0x02];
	  pf_scanline[xpos++] = lookup1[font_data & 0x01];
	}
	else {
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	}
      }
      AtariScanline (g);
      ypos++;
    }
  }

	/**
	 * This function is not currently used.
	 *
	 */
  public void antic_2 (Graphics g) {
    int nchars;
    int screendata;
    int xpos;
    int chdata, addr;
    int[] chadr = new int[48];
    int[] invert = new int[48];
    int[] blank = new int[48];

    //System.out.println("Kreslim antic 2");
    if (dlicount != 0)
      dlicount = 8;
    lookup1[0x00] = 0x0004;
    lookup1[0x80] = lookup1[0x40] = lookup1[0x20] = lookup1[0x10] =
      lookup1[0x08] = lookup1[0x04] = lookup1[0x02] = lookup1[0x01] = 0x0002;
    nchars = (xmax - xmin) >> 3;	// 40 characters per line
    for (int i = 0; i < nchars; i++) {
      screendata = memory.getByte (screenaddr++);
      chadr[i] =
	memory.chbase_40 + memory.char_offset + ((screendata & 0x7f) << 3);
      if ((screendata & memory.invert_mask) != 0)
	invert[i] = 0xff;

      else
	invert[i] = 0x00;
      if ((screendata & memory.blank_mask) != 0)
	blank[i] = 0x00;

      else
	blank[i] = 0xff;
    }
    for (int i = 0; i < xmin; i++)
      pf_scanline[i] = 0;
    for (int i = xmax; i < ATARI_WIDTH; i++)
      pf_scanline[i] = 0;
    for (int i = 0; i < 8; i++) {
      xpos = xmin;
      for (int j = 0; j < nchars; j++) {

	//      System.out.println("chadr[j]="+chadr[j]);
	addr = chadr[j];

	//  System.out.println("addr="+addr);
	chdata = memory.getByte (addr);
	chadr[j] = addr + memory.char_delta;
	chdata = (chdata ^ invert[j]) & blank[j];
	if (chdata != 0) {

	  //System.out.println("chdata="+chdata);
	  pf_scanline[xpos++] = lookup1[chdata & 0x80];
	  pf_scanline[xpos++] = lookup1[chdata & 0x40];
	  pf_scanline[xpos++] = lookup1[chdata & 0x20];
	  pf_scanline[xpos++] = lookup1[chdata & 0x10];
	  pf_scanline[xpos++] = lookup1[chdata & 0x08];
	  pf_scanline[xpos++] = lookup1[chdata & 0x04];
	  pf_scanline[xpos++] = lookup1[chdata & 0x02];
	  pf_scanline[xpos++] = lookup1[chdata & 0x01];
	}
	else {
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	  pf_scanline[xpos++] = 0x04;
	}
      }
      AtariScanline (g);
      ypos++;
    }

    //*/
  }
  public void antic_3 () {
  }

  public void antic_4 () {
  }

  public void antic_5 () {
  }

  public void antic_6 () {
  }

  public void antic_7 () {
  }

  public void antic_8 () {
  }

  public void antic_9 () {
  }

  public void antic_a () {
  }

  public void antic_b () {
  }

  public void antic_c () {
  }

  public void antic_d () {
  }

  public void antic_e () {
  }

  public void antic_f () {
  }
	/**
	 * Currently used for drawing the single Atari scanline on the 
	 * monitor. 
	 *
	 */
  public void AtariScanline (Graphics g) {
    dlicount--;
    if (dlicount == 0) {
      memory.hwreg.NMIST |= 0x80;
      memory.hwreg.VCOUNT = (ypos + 8) >> 1;
      NMI_MASK = true;
    }
    for (int i = 0; i < ATARI_WIDTH; i++) {
      switch (pf_scanline[i]) {
      case 0x00:
	g.setColor (Color.black);
	break;
      case 0x04:
	g.setColor (Color.blue);
	break;
      default:
	g.setColor (Color.white);
      }
      g.drawLine (i, ypos + 24, i, ypos + 24);
    }
  }
}
