import java.awt.*;
import java.awt.event.*;

public class AtariHardware extends Frame implements AtariConstants, KeyListener {
	AtariMem mem = new AtariMem();
	AtariCPU cpu = new AtariCPU(mem);
	Antic	antic = new Antic(mem);
	int keyCode;
	int keyChar;

	boolean stopped = false;

	public static int AKEY_NONE = 0xff;
	int key_buffer = AKEY_NONE;

	Image offImage;
	Graphics offGraphics;	


	public AtariHardware() {
		super ("Atari 800XL (130XE) Emulator");
		addKeyListener (this);
		addWindowListener ( new WindowAdapter() {
				public void windowClosing(WindowEvent we) {
				System.exit (0);
				}
				});
		setBounds (0, 0, ATARI_WIDTH, ATARI_HEIGHT+24);
		setSize (ATARI_WIDTH, ATARI_HEIGHT+24);
		setVisible (true);
	}

	public void keyPressed(KeyEvent e) {
		keyCode=e.getKeyCode();
		keyChar=e.getKeyChar();
		AtariKey();
	}

	public void keyReleased(KeyEvent e) {
	}

	public void keyTyped(KeyEvent e) {
	}


	public void update(Graphics g) {
		paint(g);
	}

	public void paint(Graphics g) {
		if (  offGraphics == null ) {
			offImage = createImage ( ATARI_WIDTH, ATARI_HEIGHT + 24 );
			offGraphics = offImage.getGraphics();
		}

		antic.UpdateScreen (offGraphics);
		g.drawImage (offImage, 0, 0, ATARI_WIDTH, ATARI_HEIGHT + 24, null);
	}

	public void AtariKey() {
		int keycode;

		switch (keyChar) {
			case '(':
				keycode=0x70;
				break;
			case ')':
				keycode=0x72;
				break;
			case '+':
				keycode=0x6;
				break;
			case '*':
				keycode=0x7;
				break;
			case '-':
				keycode=0xe;
				break;
			case '.':
				keycode=0x22;
				break;	
			case ',':
				keycode=0x20;	
				break;
			case '/':
				keycode=0x26;
				break;
			case '"':
				keycode=0x5e;
				break;
			case ';':
				keycode=0x02;
				break;
			case ':':
				keycode=0x42;
				break;
			case '#':
				keycode=0x5a;	
				break;
			case '_':
				keycode=0x4e;
				break;	
			default:
				switch (keyCode) {
					case KeyEvent.VK_ENTER:
						keycode = 0x0c;
						break;
					case KeyEvent.VK_BACK_SPACE:
						keycode = 0x34 ; 	
						break;
					case KeyEvent.VK_SPACE:
						keycode = 0x21;
						break;
					case KeyEvent.VK_EQUALS:
						keycode = 0x0f;
						break;
					case KeyEvent.VK_0 :
						keycode = 0x32;
						break;				
					case KeyEvent.VK_1 :
						keycode = 0x1f;
						break;				
					case KeyEvent.VK_2 :
						keycode = 0x1e;
						break;				
					case KeyEvent.VK_3 :
						keycode = 0x1a;
						break;				
					case KeyEvent.VK_4 :
						keycode = 0x18;
						break;				
					case KeyEvent.VK_5 :
						keycode = 0x1d;
						break;				
					case KeyEvent.VK_6 :
						keycode = 0x1b;
						break;				
					case KeyEvent.VK_7 :
						keycode = 0x33;
						break;				
					case KeyEvent.VK_8 :
						keycode = 0x35;
						break;				
					case KeyEvent.VK_9 :
						keycode = 0x30;
						break;				
					case KeyEvent.VK_A :
						keycode = 0x3f;
						break;
					case KeyEvent.VK_B :
						keycode = 0x15;
						break;
					case KeyEvent.VK_C :
						keycode = 0x12;
						break;
					case KeyEvent.VK_D :
						keycode = 0x3a;
						break;
					case KeyEvent.VK_E :
						keycode = 0x2a;
						break;
					case KeyEvent.VK_F :
						keycode = 0x38;
						break;
					case KeyEvent.VK_G :
						keycode = 0x3d;
						break;
					case KeyEvent.VK_H :
						keycode = 0x39;
						break;
					case KeyEvent.VK_I :
						keycode = 0x0d;
						break;
					case KeyEvent.VK_J :
						keycode = 0x01;
						break;
					case KeyEvent.VK_K :
						keycode = 0x05;
						break;
					case KeyEvent.VK_L :
						keycode = 0x00;
						break;
					case KeyEvent.VK_M :
						keycode = 0x25;
						break;
					case KeyEvent.VK_N :
						keycode = 0x23;
						break;
					case KeyEvent.VK_O :
						keycode = 0x08;
						break;
					case KeyEvent.VK_P :
						keycode = 0x0a;
						break;
					case KeyEvent.VK_Q :
						keycode = 0x2f;
						break;
					case KeyEvent.VK_R :
						keycode = 0x28;
						break;
					case KeyEvent.VK_S :
						keycode = 0x3e;
						break;
					case KeyEvent.VK_T :
						keycode = 0x2d;
						break;
					case KeyEvent.VK_U :
						keycode = 0x0b;
						break;
					case KeyEvent.VK_V :
						keycode = 0x10;
						break;
					case KeyEvent.VK_W :
						keycode = 0x2e;
						break;
					case KeyEvent.VK_X :
						keycode = 0x16;
						break;
					case KeyEvent.VK_Y :
						keycode = 0x2b;
						break;
					case KeyEvent.VK_Z :
						keycode = 0x17;
						break;
					case KeyEvent.VK_COMMA:
						keycode = 0x20;
						break;
					case KeyEvent.VK_OPEN_BRACKET:
						keycode=0x60;
						break;
					case KeyEvent.VK_CLOSE_BRACKET:
						keycode=0x62;
						break;
					default:
						keycode = AKEY_NONE;
				}
		}

		key_buffer = keycode;
	}

	/**
	 * The main routine for the Atari Emulator
	 */     

	public void start() {
		int keycode;

		stopped = false;

		while (!stopped) {
			cpu.memory.hwreg.NMIST = 0x00;

			/* Debug levels : */
			cpu.trace_jumps = false;
			cpu.trace = false;

			keycode = key_buffer;

			if (keycode != AKEY_NONE) {
				// System.out.println("AtariKey() : "+keycode);

				key_buffer = AKEY_NONE;
				cpu.memory.hwreg.KBCODE = keycode & 0xff;
				cpu.memory.hwreg.IRQST = 0xbf;
				cpu.IRQ = true;
				//cpu.trace = true;
			}

			if ( (cpu.memory.hwreg.NMIEN & 0x40) != 0) {
				cpu.memory.hwreg.NMIST |= 0x40;
				cpu.NMI = true;
				//	System.out.println("NMI @ AtariHardware");
			}

			cpu.decodeInstruction(4000);

			//System.out.println("Dalsich 4000 uspesne vykonanych instrukci");
			repaint();
			cpu.NMI = antic.NMI_MASK;
		}
	}
}
