import java.io.*;

public class Memory {
	public  int MAX_MEM ;
	public final static int ROM = 1;
	public final static int RAM = 2;


	public int[] mem ;
	public int[] attrib ;
	

	/**
	 * Initilizes the main memory.
	 */
	public Memory() {
		this(0x10000);
	}
		

	public Memory(int size) {
		MAX_MEM = size;
			
		mem = new int[MAX_MEM];
		attrib = new int[MAX_MEM];
				
		for (int i=0; i<MAX_MEM; i++) {
			mem[i] = 0;
			attrib[i] = RAM;
		}
	}
	  
	  
	/**
	 * This method write the value what into the memory location specified
	 * by the where value.
	 *
	 * @param   where  where to store the value
	 * @param   what   what to store in the memory
	 */
	public void writeMem(int where, int what) {
		mem[where & 0xffff] = what & 0xff;
	}
		

	/**
	 *		We use this function when we want to read from the CPU memory,
	 *   which is certainly private.
	 *		
	 * @param   where  location from where to read
	 * @return     		 the value from the memory
	 */
	public int readMem(int where) {
		return mem[where & 0xffff] & 0xff;
	}
		

	/**
	 * This method is similar to writeMem method but the there is
	 * a difference between accessing the memory. The writeMem method
	 * allows you to write anywhere you want. The putByte method reflects
	 * the installed memory. So you can't write into the ROM with
	 * this method. You must use the writeMem method to do this.
	 *
	 * @param   where  where to store the value
	 * @param   what   what to store in the memory
	 */
		
		
	public void putByte(int where, int what) {
		if (attrib[where] != ROM) 			// You cannot write to ROM
			mem[where] = what & 0xff;
	}
		
	/**
	 * This method is the same like readMem() with the diffrence that
	 * it doesn't test if you are in memory range.
	 * @param   where  location from where to read
	 * @return     		 the value from the memory
	 */
	public int getByte(int where) {
		return mem[where] & 0xff;
	}
		
		

	/**
	 *	 Uses getByte() method	to extract word from the memory location
	 *
	 * @param   where  from where to extract the word value
	 * @return     word value (returned in int type)
	 */
	public int getWord(int where) {
		return (getByte(where+1) << 8) | getByte(where);
	}
	  
	  
	/**
	 * Tells the Atari computer emulator where is the ROM location.
	 *
	 * @param   from  start of the ROM location
	 * @param   to  end of the ROM location
	 */
	public void setROM(int from, int to) {
		for (int i=from; i<=to; i++) 
			attrib[i] = ROM;
	}
		
	/**
	 * Tells the Atari computer emulator where is the RAM location.
	 *
	 * @param   from  start of the RAM location
	 * @param   to  end of the RAM location
	 */
		
	public void setRAM(int from, int to) {
		for (int i=from; i<=to; i++) 
			attrib[i] = RAM;
	}

	

	/**
	 * Installs the binary image into the memory. We use this to load
	 * Atari OS, Atari Basic into the specified locations.
	 *
	 * @param   from  begining address
	 * @param   to    finish address
	 * @param   filename  filename representing the binary image (OS, Basic,...)
	 */
	public void  installDataBlock(int from, int to, String filename) throws IOException {
		FileInputStream istream;
				
		int b = 0;
				
		try {
			istream = new FileInputStream (filename);
		}
				
		catch (IOException e) {
			System.out.println("Can't load filename:"+filename);
			throw new IOException();
		}
				
		System.out.println("Loading "+filename+" from: "+from+"  to: "+to);

		for (int i=from; i<=to; i++) {
			try {	
				b = istream.read();
			}
			catch (IOException e) {
				System.out.println("Read failure in file"+filename);
				throw new IOException();
			}
					 
			mem[i] = b;					
		}
	
	}
		
}
		

